<?php 
// AJAX 评论提交部分
if(!defined('THEME_NAME')) die('非法访问 - Insufficient Permissions');

// 输出评论错误信息
function mk_ajax_comment_err($msg) {
    header('HTTP/1.1 405 Method Not Allowed');
    header('Content-Type: text/plain;charset=UTF-8');
    die($msg);
}

// 禁止无中文留言
if ( !is_user_logged_in() ) {
    function mk_refused_no_Chinese_comment( $comment_data ) {
        $pattern  = '/[一-龥]/u'; 
        $jpattern = '/[ぁ-ん]+|[ァ-ヴ]+/u';	
        
        // 禁止无中文评论
        if(!preg_match($pattern, $comment_data['comment_content'])) {
            mk_ajax_comment_err('Can U speak Chinese?');
        }
        
        // 禁止包含日文
        if (preg_match($jpattern, $comment_data['comment_content'])) {
            mk_ajax_comment_err('禁止使用日文评论！');
        }
        return($comment_data);
    }
    add_filter('preprocess_comment','mk_refused_no_Chinese_comment');
}

// 评论内容黑名单检测
function mk_comment_blacklist_check($comment) {
    if (wp_blacklist_check(
        $comment['comment_author'], 
        $comment['comment_author_email'], 
        $comment['comment_author_url'], 
        $comment['comment_content'], 
        $comment['comment_author_IP'], 
        $comment['comment_agent'])) {
        mk_ajax_comment_err('您的评论中包含不受欢迎的内容，请修改');
    } else {
        return $comment;
    }
}
add_filter('preprocess_comment', 'mk_comment_blacklist_check');

// 评论 ajax 提交
function mk_ajax_comment_callback() {
    global $wpdb;
    
    if(!isset($_POST['comment_post_ID'])) mk_ajax_comment_err('该文章不存在');
    
    $comment_post_ID = (int) $_POST['comment_post_ID'];
    $post = get_post($comment_post_ID);
    if ( empty($post->comment_status) ) {
        do_action('comment_id_not_found', $comment_post_ID);
        mk_ajax_comment_err('该文章不存在');
    }
    
    // 判断文章状态
    $status     = get_post_status($post);
    $status_obj = get_post_status_object($status);
    if ( !comments_open($comment_post_ID) ) {
        do_action('comment_closed', $comment_post_ID);
        mk_ajax_comment_err('对不起，本文评论已关闭');
    } elseif ( 'trash' == $status ) {
        do_action('comment_on_trash', $comment_post_ID);
        mk_ajax_comment_err('未知错误');
    } elseif ( !$status_obj->public && !$status_obj->private ) {
        do_action('comment_on_draft', $comment_post_ID);
        mk_ajax_comment_err('该文章不存在');
    } elseif ( post_password_required($comment_post_ID) ) {
        do_action('comment_on_password_protected', $comment_post_ID);
        mk_ajax_comment_err('密码保护中');
    } else {
        do_action('pre_comment_on_post', $comment_post_ID);
    }
    
    // 获取评论内容
    $comment_content = ( isset($_POST['comment']) ) ? trim($_POST['comment']) : null;
    
    // 评论不能为空
    if ( !$comment_content ) mk_ajax_comment_err( '说点什么吧？' );
    
    // 获取评论者信息
    $user = wp_get_current_user();
    if ( $user->exists() ) {
        // 已登录用户
        if ( empty( $user->display_name ) ) $user->display_name=$user->user_login;
        $comment_author       = esc_sql($user->display_name);
        $comment_author_email = esc_sql($user->user_email);
        $comment_author_url   = esc_sql($user->user_url);
        $user_ID              = esc_sql($user->ID);
        
        if ( current_user_can('unfiltered_html') ) {
            // 防止 xss
            if ( wp_create_nonce('unfiltered-html-comment_' . $comment_post_ID) != $_POST['_wp_unfiltered_html_comment'] ) {
                kses_remove_filters();
                kses_init_filters();
            }
        }
    } else {
        // 未登录用户
        if ( get_option('comment_registration') || 'private' == $status ) mk_ajax_comment_err('抱歉，您必须登录后才能发表评论');
        
        // 获取评论者填写的信息
        $comment_author       = isset($_POST['author']) ? trim(strip_tags($_POST['author'])) : null;
        $comment_author_email = isset($_POST['email'])  ? trim($_POST['email'])              : null;
        $comment_author_url   = isset($_POST['url'])    ? trim($_POST['url'])                : null;
        
        // 判断信息是否填写完整，正确
        if(!$comment_author) mk_ajax_comment_err('请填写昵称');
        if(get_option('require_name_email') && !$comment_author_email) mk_ajax_comment_err('请填写邮箱');
        if(!is_email($comment_author_email)) mk_ajax_comment_err('请输入有效的电子邮件地址');
        
        // 防注入
        $comment_author       = esc_sql($comment_author);
        $comment_author_email = esc_sql($comment_author_email);
        $comment_author_url   = esc_sql($comment_author_url);
        
        // 防止冒充管理评论
        if($comment_author_email == get_option('admin_email')) mk_ajax_comment_err('管理员请登录后再发表评论！');
        
        // 防止刷评论
        if($lasttime = $wpdb->get_var( $wpdb->prepare("SELECT comment_date_gmt FROM $wpdb->comments WHERE comment_author = %s ORDER BY comment_date DESC LIMIT 1", $comment_author) ) ) {
            $time_lastcomment = mysql2date('U', $lasttime, false);
            $time_newcomment  = mysql2date('U', current_time('mysql', 1), false);
            $flood_die        = apply_filters('comment_flood_filter', false, $time_lastcomment, $time_newcomment);
            if ($flood_die) {
                mk_ajax_comment_err('您回复太快啦。喝杯茶歇会吧~');
            }
        }
        
        // 防止重复提交
        $dupe = "SELECT comment_ID FROM $wpdb->comments WHERE comment_post_ID = '$comment_post_ID' AND ( comment_author = '$comment_author' ";
        if ( $comment_author_email ) $dupe .= "OR comment_author_email = '$comment_author_email' ";
        $dupe .= ") AND comment_content = '$comment_content' LIMIT 1";
        if ( $wpdb->get_var($dupe) ) {
            mk_ajax_comment_err('该评论已存在，请不要重复提交评论！');
        }
    }
    
    // 获取父评论 ID
    $comment_parent = isset($_POST['comment_parent']) ? absint($_POST['comment_parent']) : 0;
    
    // 评论入库
    $commentdata = compact('comment_post_ID', 'comment_author', 'comment_author_email', 'comment_author_url', 'comment_content', 'comment_type', 'comment_parent', 'user_ID');

    // 从数据库中拿出评论内容
    $comment_id = wp_new_comment( $commentdata );
    $comment    = get_comment($comment_id);
    
    // 记录 Cookie
    if ( !$user->ID ) {
        $comment_cookie_lifetime = apply_filters('comment_cookie_lifetime', 30000000);
        setcookie('comment_author_' .       COOKIEHASH, $comment->comment_author,       time() + $comment_cookie_lifetime, COOKIEPATH, COOKIE_DOMAIN);
        setcookie('comment_author_email_' . COOKIEHASH, $comment->comment_author_email, time() + $comment_cookie_lifetime, COOKIEPATH, COOKIE_DOMAIN);
        setcookie('comment_author_url_' .   COOKIEHASH, $comment->comment_author_url,   time() + $comment_cookie_lifetime, COOKIEPATH, COOKIE_DOMAIN);
    }
    
    // 调用主题内置评论模板输出评论预览
    mytheme_comment($comment, '', $comment_parent);
    
    die();
}
add_action('wp_ajax_nopriv_ajax_comment', 'mk_ajax_comment_callback');
add_action('wp_ajax_ajax_comment',        'mk_ajax_comment_callback');

?>